@@ -1,6 +1,6 @@
 /* ptp-private.h
  *
- * Copyright (C) 2006 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (C) 2011 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,16 +33,9 @@ void report_result (GPContext *context, short result, short vendor);
 int translate_ptp_result (short result);
 void fixup_cached_deviceinfo (Camera *camera, PTPDeviceInfo*);
 
-int init_ptp_fs (Camera *camera, GPContext *context);
-
-int _ptp_check_eos_events (PTPParams *params);
-int _ptp_get_one_eos_event (PTPParams *params, PTPCanon_changes_entry *entry);
-
-
 struct _CameraPrivateLibrary {
 	PTPParams params;
-	unsigned long bugs;
-	int	fs_loaded;
+	int checkevents;
 };
 
 struct _PTPData {
