@@ -338,7 +338,7 @@ sierra_get_memory_left (Camera *camera, int *memory, GPContext *context)
 static int
 sierra_check_connection (Camera *camera, GPContext *context)
 {
-	int r = 0, timeout;
+	int r = 0, ret, timeout;
 	unsigned char c;
 
 	/* Only serial cameras close the connection after some timeout. */
@@ -354,9 +354,9 @@ sierra_check_connection (Camera *camera, GPContext *context)
 		 */
 		CHECK (gp_port_get_timeout (camera->port, &timeout));
 		CHECK (gp_port_set_timeout (camera->port, 20));
-		r = gp_port_read (camera->port, &c, 1);
+		ret = gp_port_read (camera->port, &c, 1);
 		CHECK (gp_port_set_timeout (camera->port, timeout));
-		switch (r) {
+		switch (ret) {
 		case GP_ERROR_TIMEOUT:
 		case GP_ERROR_IO_READ:
 
@@ -372,7 +372,7 @@ sierra_check_connection (Camera *camera, GPContext *context)
 			 * If any error (except timeout) has occurred, 
 			 * report it.
 			 */
-			CHECK (r);
+			CHECK (ret);
 		}
 
 		/*
@@ -524,6 +524,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 	GP_DEBUG ("Reading packet...");
 
 	switch (camera->port->type) {
+	case GP_PORT_USB_SCSI:
 	case GP_PORT_USB:
 		blocksize = SIERRA_PACKET_SIZE;
 		break;
@@ -545,7 +546,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		 * Read data through the bus. If an error occurred,
 		 * try again.
 		 */
-		if (camera->port->type == GP_PORT_USB && (camera->pl->flags & SIERRA_WRAP_USB_MASK))
+		if ((camera->port->type == GP_PORT_USB_SCSI) && (camera->pl->flags & SIERRA_WRAP_USB_MASK))
 			result = usb_wrap_read_packet (camera->port,
 					(camera->pl->flags & SIERRA_WRAP_USB_MASK),
 					packet, blocksize);
@@ -562,6 +563,16 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			GP_DEBUG ("Retrying...");
 			continue;
 		}
+		if (result == 0) {
+			GP_DEBUG ("Read got 0 bytes.."); 
+			if (++r > 2) {
+				sierra_clear_usb_halt(camera);
+				GP_DEBUG ("Giving up...");
+				return GP_ERROR_IO_READ;
+			}
+			GP_DEBUG ("Retrying...");
+			continue;
+		}
 		br = result;
 
 		/*
@@ -580,7 +591,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 			/* Those are all single byte packets. */
 			sierra_clear_usb_halt(camera);
-			GP_DEBUG ("Packet read. Returning GP_OK.");
+			GP_DEBUG ("Packet type 0x%02x read. Returning GP_OK.", packet[0]);
 			return GP_OK;
 
 		case SIERRA_PACKET_DATA:
@@ -748,6 +759,7 @@ sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 
 		/* Write packet and read the answer */
 		CHECK (sierra_write_packet (camera, packet, context));
+		buf[0] = 0;
 		result = sierra_read_packet_wait (camera, buf, context);
 		switch (result) {
 		case GP_ERROR_CORRUPTED_DATA:
@@ -999,7 +1011,8 @@ int sierra_sub_action (Camera *camera, SierraAction action, int sub_action,
 	CHECK (sierra_read_packet_wait (camera, buf, context));
 
 	switch (buf[0]) {
-		case SIERRA_PACKET_ENQ:
+	case ACK: /* reported on capture on Olympus 4040 */
+	case SIERRA_PACKET_ENQ:
 		return (GP_OK);
 	default:
 		gp_context_error (context, _("Received unexpected answer "
@@ -1059,6 +1072,7 @@ int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *con
 	while (1) {
 
 		/* Read the response */
+		buf[0] = 0;
 		CHECK (sierra_read_packet_wait (camera, buf, context));
 		GP_DEBUG ("Successfully read packet. Interpreting result "
 			  "(0x%02x)", buf[0]);
@@ -1195,7 +1209,6 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 	int retries, r;
 	unsigned int min_progress_bytes;
 	static int in_function = 0;
-	const char *file_name;
 	unsigned int id = 0;
 
 	GP_DEBUG ("sierra_get_string_register:  reg %i, file number %i, "
@@ -1232,7 +1245,6 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 	CHECK (sierra_write_packet (camera, p, context));
 
 	if (file && total > min_progress_bytes) {
-		CHECK (gp_file_get_name(file, &file_name));
 		id = gp_context_progress_start (context, total, _("Downloading data..."));
 	}
 
@@ -1380,7 +1392,10 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 		 * current picture.
 		 */ 
 		GP_DEBUG ("Getting picture number.");
-		CHECK (sierra_get_int_register (camera, 4, &n, context));
+		r = sierra_get_int_register (camera, 4, &n, context);
+		if (r == GP_OK) {
+			GP_DEBUG ("Getting filename of file %i.", n);
+		}
 		/*
 		 * We need to tell the frontend where the new image can be
 		 * found.  Unfortunatelly, we can only figure out the
@@ -1392,8 +1407,10 @@ sierra_capture (Camera *camera, CameraCaptureType type,
 		 *
 		 * Not that some cameras that don't support filenames will
 		 * return 8 blanks instead of reporting an error.
+		 * 
+		 * Some newer cameras (Nikon*) do not return register 4, so
+		 * ignore errors from that.
 		 */
-		GP_DEBUG ("Getting filename of file %i.", n);
 		CHECK (sierra_get_string_register (camera, 79, 0, NULL,
 						   filename, &len, context));
 		if ((len <= 0) || !strcmp (filename, "        "))
