@@ -17,6 +17,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#define _BSD_SOURCE
+
 #include "config.h"
 #include "sierra.h"
 
@@ -205,11 +207,17 @@ static struct {
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "fe-200", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
-
+/*
+	{"Olympus", "E-520", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x110,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+*/
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0, NULL },
 
 	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX, NULL },
 	{"Pentax", "Optio 33WR", SIERRA_MODEL_DEFAULT, 0x0a17,0x0009, SIERRA_WRAP_USB_PENTAX, NULL },
+	/* has only very bare reactions
+	{"Pentax", "Optio W90 (Sierra Mode)", SIERRA_MODEL_DEFAULT, 0x0a17,0x00f6, SIERRA_WRAP_USB_PENTAX, NULL },
+	 */
 
 	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
@@ -269,8 +277,12 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.status = GP_DRIVER_STATUS_PRODUCTION;
 		a.port     = GP_PORT_SERIAL;
 		if ((sierra_cameras[x].usb_vendor  > 0) &&
-		    (sierra_cameras[x].usb_product > 0))
-			a.port |= GP_PORT_USB;
+		    (sierra_cameras[x].usb_product > 0)) {
+			if (sierra_cameras[x].flags & (SIERRA_WRAP_USB_OLYMPUS|SIERRA_WRAP_USB_PENTAX|SIERRA_WRAP_USB_NIKON))
+			    a.port |= GP_PORT_USB_SCSI;
+                        else
+			    a.port |= GP_PORT_USB;
+		}
 		a.speed[0] = 9600;
 		a.speed[1] = 19200;
 		a.speed[2] = 38400;
@@ -350,11 +362,6 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	info->audio.fields   = GP_FILE_INFO_NONE;
 	info->file.permissions = GP_FILE_PERM_READ;
 
-	/* Name of image */
-	strncpy (info->file.name, filename, sizeof (info->file.name) - 1);
-	info->file.name[sizeof (info->file.name) - 1] = '\0';
-	info->file.fields |= GP_FILE_INFO_NAME;
-
 	/*
 	 * Get information about this image. Don't make this fatal as
 	 * at least the "Agfa ePhoto307" doesn't support this command.
@@ -477,6 +484,7 @@ camera_start (Camera *camera, GPContext *context)
 		break;
 
 	case GP_PORT_USB:
+	case GP_PORT_USB_SCSI:
 		CHECK (gp_port_set_timeout (camera->port, 5000));
 		break;
 	default:
@@ -733,22 +741,22 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 }
 
 static int
-put_file_func (CameraFilesystem * fs, const char *folder, CameraFile * file, void *data, GPContext *context)
+put_file_func (CameraFilesystem * fs, const char *folder, const char *filename,
+	CameraFileType type, CameraFile * file, void *data, GPContext *context)
 {
 	Camera *camera = data;
 	char *picture_folder;
 	int ret;
 	const char *data_file;
 	long data_size;
-	const char *filename;
 	int available_memory;
 
-	gp_file_get_name(file, &filename);
-	
-
 	GP_DEBUG ("*** put_file_func");
 	GP_DEBUG ("*** folder: %s", folder);
 	GP_DEBUG ("*** filename: %s", filename);
+
+	if (type != GP_FILE_TYPE_NORMAL)
+		return GP_ERROR_BAD_PARAMETERS;
 	
 	/* Check the size */
 	CHECK (gp_file_get_data_and_size (file, &data_file, &data_size));
@@ -2230,6 +2238,7 @@ camera_init (Camera *camera, GPContext *context)
                 break;
 
         case GP_PORT_USB:
+        case GP_PORT_USB_SCSI:
 
                 /* Test if we have usb information */
                 if (vendor == 0) {
@@ -2272,6 +2281,10 @@ camera_init (Camera *camera, GPContext *context)
         /* FIXME??? What's that for? "Resetting folder system"? */
 	sierra_set_int_register (camera, 83, -1, NULL);
 #endif
+	/* How to switch the coolpix 2500 between RAW / NORMAL
+   return dsc->SetStr(0x16, raw_enabled ? "DIAG RAW" : "NIKON DIGITAL CAMERA" );
+        ret = sierra_set_string_register (camera, 0x16, "NIKON DIGITAL CAMERA", strlen("NIKON DIGITAL CAMERA"), NULL);
+	 */
 
         /* Folder support? */
 	CHECK_STOP_FREE (camera, gp_port_set_timeout (camera->port, 50));
