@@ -16,6 +16,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+#define _BSD_SOURCE
+#define _POSIX_C_SOURCE 1
 #include "config.h"
 
 #include <stdio.h>
@@ -820,7 +822,10 @@ st2205_init(Camera *camera)
 		  ST2205_V2_PICTURE_START, 1 },
 		{ }
 	};
-	const int uncompressed_firmware_checksums[] = { 0x00ab02fc, 0 };
+	const int uncompressed_firmware_checksums[] = {
+	        0x00ab02fc, /* Frame 96x64 from blokker (Netherlands) */
+	        0x00aa8060, /* Blokker frame with picframe hacked firmware */
+	        0 };
 
 	GP_DEBUG ("st2205_init called");
 
